/* mpiv0.c - functions to read an MPIV0 file */


#include <stdio.h>
#include <stdlib.h>
#include "mpiv0.h"


#define		YES		-1
#define		NO		0

#define		UCHAR		unsigned char

#define MIN(x, y)  (((x) < (y)) ? (x) : (y))
#define MAX(x, y)  (((x) > (y)) ? (x) : (y))

static char mpiv_getc( FILE *file );
static int mpiv_getint( FILE *file );





/* mpiv0_open() - read the header information from an MPIV0 file,
               return file pointer if successful, NULL if error */

FILE *mpiv0_open( char *name, int *xsize, int *ysize,
		    MPIV0_FormatIndex *format )
{
  FILE *fp;
  int i;
  char magic[6];

  /* open mpiv input file */
  if ( (fp = fopen( name, "r" )) == NULL )
    return NULL;

  /* read and test magic number */
  for( i = 0; i < 5; i++ )
    magic[i] = fgetc( fp );
  magic[5] = 0;
  if ( strcmp( magic, "MPIV0" ) != 0 )
    return NULL;

  *xsize = mpiv_getint( fp );
  *ysize = mpiv_getint( fp );
  *format = mpiv_getint( fp );

  return fp;

}





/* mpiv0_read() - reads a frame from an MPIV0 file
   returns number of frames read (0 = failure, 1 otherwise) */

int mpiv0_read( FILE *fp, int xsize, int ysize, MPIV0_FormatIndex format,
	       UCHAR **r, UCHAR **g, UCHAR **b )
{
  int i;
  int npix;
  int pixel16, lo, hi, rgb;
  UCHAR *inbuffer, *p;

  npix = xsize * ysize;

  switch ( format ) {
  case RGB24 :
    inbuffer = (unsigned char *)malloc( npix * sizeof(unsigned char) * 3 );
    if ( fread( inbuffer, sizeof(unsigned char), npix*3, fp ) != npix * 3 ) {
      free( inbuffer );
      return 0;
    }
    *r = (UCHAR *)malloc( npix * sizeof(unsigned char) );
    *g = (UCHAR *)malloc( npix * sizeof(unsigned char) );
    *b = (UCHAR *)malloc( npix * sizeof(unsigned char) );
    for( i = 0; i < npix; i++ ) {
      (*r)[i] = inbuffer[3*i];
      (*g)[i] = inbuffer[3*i+1];
      (*b)[i] = inbuffer[3*i+2];
    }
    free( inbuffer );
    return 1;
  case RGB15 :
    inbuffer = (unsigned char *)malloc( npix * sizeof(unsigned char) * 2 );
    if ( fread( inbuffer, sizeof(unsigned char), npix*2, fp ) != npix * 2 ) {
      free( inbuffer );
      return 0;
    }
    *r = (UCHAR *)malloc( npix * sizeof(unsigned char) );
    *g = (UCHAR *)malloc( npix * sizeof(unsigned char) );
    *b = (UCHAR *)malloc( npix * sizeof(unsigned char) );
    for( i = 0, p = inbuffer; i < npix; i++ ) {
      lo = *p++; hi = *p++;
      pixel16 = lo + (hi << 8);
      (*r)[i] = (pixel16 & 0x7c00) >> 7;
      (*g)[i] = (pixel16 & 0x03e0) >> 2;
      (*b)[i] = (pixel16 & 0x001f) << 3;
    }
    free( inbuffer );
    return 1;
  case RGB8 :
    inbuffer = (unsigned char *)malloc( npix * sizeof(unsigned char) * 1 );
    if ( fread( inbuffer, sizeof(unsigned char), npix, fp ) != npix ) {
      free( inbuffer );
      return 0;
    }
    *r = (unsigned char *)malloc( npix * sizeof(unsigned char) );
    *g = (unsigned char *)malloc( npix * sizeof(unsigned char) );
    *b = (unsigned char *)malloc( npix * sizeof(unsigned char) );
    for( i = 0, p = inbuffer; i < npix; i++ ) {
      rgb = *p++;
      (*r)[i] =  rgb & 0xe0;
      (*g)[i] = (rgb & 0x1c) << 3;
      (*b)[i] = (rgb & 0x03) << 6;
    }
    free( inbuffer );
    return 1;
  case BW8 :
    inbuffer = (unsigned char *)malloc( npix * sizeof(unsigned char) * 1 );
    if ( fread( inbuffer, sizeof(unsigned char), npix, fp ) != npix ) {
      free( inbuffer );
      return 0;
    }
    *r = *g = *b = (unsigned char *)malloc( npix * sizeof(unsigned char) );
    for( i = 0, p = inbuffer; i < npix; i++ ) {
      (*r)[i] = *p++;
    }
    free( inbuffer );
    return 1;
  default :
    return 0;
  }
  return 0;
}





/* ------------------------------------------------------------------
The following code is adapted from netpbm package by Jef Poskanzer.

	Copyright (C) 1988 by Jef Poskanzer.
--------------------------------------------------------------------- */


static char mpiv_getc( FILE *file )
{
  int ich;
  char ch;

  ich = getc( file );
  if ( ich == EOF ) return (char) EOF;
  ch = (char) ich;
    
  if ( ch == '#' ) {
    do {
      ich = getc( file );
      if ( ich == EOF ) return (char) EOF;
      ch = (char) ich;
    } while ( ch != '\n' && ch != '\r' );
  }
  return ch;
}


static int mpiv_getint( FILE *file )
{
  char ch;
  int i;

  do {
    ch = mpiv_getc( file );
  } while ( ch == ' ' || ch == '\t' || ch == '\n' || ch == '\r' );

  if ( ch < '0' || ch > '9' ) return -1;

  i = 0;
  do {
    i = i * 10 + ch - '0';
    ch = mpiv_getc( file );
  } while ( ch >= '0' && ch <= '9' );

  return i;
}




